"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var parser_1 = require("./parser");
var Context = (function () {
    function Context(processor) {
        this.processor = processor;
        this.renderStack = [];
    }
    Context.prototype.getParser = function (parserId) {
        var def = this.processor.getParserDef(parserId);
        return this.createParser(def);
    };
    Context.prototype.getMainParser = function () {
        var def = this.processor.getMainParserDef();
        return this.createParser(def);
    };
    Context.prototype.createParser = function (def) {
        var rules = def(this);
        return new parser_1.Parser(this, rules);
    };
    Context.prototype.render = function (node) {
        var res = '';
        var nodes = this.runTransforms(node);
        for (var _i = 0, nodes_1 = nodes; _i < nodes_1.length; _i++) {
            var node_1 = nodes_1[_i];
            res += node_1.render(this);
        }
        return res;
    };
    Context.prototype.renderChildren = function (node) {
        var res = '';
        this.renderStack.push(node);
        for (var _i = 0, _a = node.children; _i < _a.length; _i++) {
            var child = _a[_i];
            res += this.render(child);
        }
        this.renderStack.pop();
        return res;
    };
    Context.prototype.runTransforms = function (node) {
        var nodes = [node];
        for (var _i = 0, _a = this.processor.transforms; _i < _a.length; _i++) {
            var transform = _a[_i];
            var res = [];
            for (var _b = 0, nodes_2 = nodes; _b < nodes_2.length; _b++) {
                var node_2 = nodes_2[_b];
                var newNodes = transform(node_2, this);
                res.push.apply(res, newNodes);
            }
            nodes = res;
        }
        return nodes;
    };
    return Context;
}());
exports.Context = Context;
//# sourceMappingURL=context.js.map