"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var region_1 = require("./region");
var constants = __importStar(require("./constants"));
var RANGE_DIGIT_START = constants.RANGE_DIGIT_START, RANGE_DIGIT_END = constants.RANGE_DIGIT_END, RANGE_LATIN_LOWER_START = constants.RANGE_LATIN_LOWER_START, RANGE_LATIN_LOWER_END = constants.RANGE_LATIN_LOWER_END, RANGE_LATIN_UPPER_START = constants.RANGE_LATIN_UPPER_START, RANGE_LATIN_UPPER_END = constants.RANGE_LATIN_UPPER_END, CHAR_SPACE = constants.CHAR_SPACE, CHAR_TAB = constants.CHAR_TAB, CHAR_LF = constants.CHAR_LF, CHAR_FF = constants.CHAR_FF, CHAR_CR = constants.CHAR_CR, CHAR_MINUS = constants.CHAR_MINUS, CHAR_UNDERSCORE = constants.CHAR_UNDERSCORE, CHAR_BACKSLASH = constants.CHAR_BACKSLASH;
var Cursor = (function () {
    function Cursor(source, _pos) {
        if (_pos === void 0) { _pos = 0; }
        this._pos = _pos;
        this.region = source instanceof region_1.Region ? source : new region_1.Region(source);
    }
    Object.defineProperty(Cursor.prototype, "pos", {
        get: function () {
            return this._pos;
        },
        enumerable: true,
        configurable: true
    });
    Cursor.prototype.set = function (i) {
        this._pos = i;
        return this;
    };
    Cursor.prototype.clone = function () {
        return new Cursor(this.region, this.pos);
    };
    Cursor.prototype.peek = function (offset) {
        if (offset === void 0) { offset = 1; }
        return this.region.charAt(this.pos + offset);
    };
    Cursor.prototype.peekCode = function (offset) {
        if (offset === void 0) { offset = 1; }
        return this.region.charCodeAt(this.pos + offset);
    };
    Cursor.prototype.current = function () {
        return this.region.charAt(this.pos);
    };
    Cursor.prototype.currentCode = function () {
        return this.region.charCodeAt(this.pos);
    };
    Cursor.prototype.hasCurrent = function () {
        return this.pos < this.region.length;
    };
    Cursor.prototype.subRegion = function (start, end) {
        if (end === void 0) { end = this.region.length; }
        return this.region.subRegion(start, end);
    };
    Cursor.prototype.at = function (str) {
        var end = this.pos + str.length;
        if (end > this.region.length) {
            return false;
        }
        for (var i = 0; i < str.length; i++) {
            if (str[i] !== this.region.charAt(this.pos + i)) {
                return false;
            }
        }
        return true;
    };
    Cursor.prototype.atCode = function (code, offset) {
        if (offset === void 0) { offset = 0; }
        return this.peekCode(offset) === code;
    };
    Cursor.prototype.atSeq = function () {
        var seq = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            seq[_i] = arguments[_i];
        }
        for (var i = 0; i < seq.length; i++) {
            if (seq[i] !== this.peekCode(i)) {
                return false;
            }
        }
        return true;
    };
    Cursor.prototype.atInsensitive = function (str) {
        var end = this.pos + str.length;
        if (end > this.region.length) {
            return false;
        }
        return this.region.substring(this.pos, end).toLowerCase() === str.toLowerCase();
    };
    Cursor.prototype.atDigit = function () {
        var c = this.currentCode();
        return c >= RANGE_DIGIT_START && c <= RANGE_DIGIT_END;
    };
    Cursor.prototype.atLatin = function () {
        var c = this.currentCode();
        return c >= RANGE_LATIN_LOWER_START && c <= RANGE_LATIN_LOWER_END ||
            c >= RANGE_LATIN_UPPER_START && c <= RANGE_LATIN_UPPER_END;
    };
    Cursor.prototype.atIdentifier = function () {
        var c = this.currentCode();
        return c >= RANGE_LATIN_LOWER_START && c <= RANGE_LATIN_LOWER_END ||
            c >= RANGE_LATIN_UPPER_START && c <= RANGE_LATIN_UPPER_END ||
            c >= RANGE_DIGIT_START && c <= RANGE_DIGIT_END ||
            c === CHAR_MINUS || c === CHAR_UNDERSCORE;
    };
    Cursor.prototype.atNewLine = function () {
        var c = this.currentCode();
        return c === CHAR_LF || c === CHAR_CR;
    };
    Cursor.prototype.atBlankLine = function () {
        var p = this.pos;
        this.skipSpaces();
        var result = this.atNewLine() || !this.hasCurrent();
        this.set(p);
        return result;
    };
    Cursor.prototype.atSpace = function () {
        var c = this.currentCode();
        return c === CHAR_SPACE || c === CHAR_TAB;
    };
    Cursor.prototype.atSpaces = function (count) {
        for (var i = 0; i < count; i++) {
            var c = this.peekCode(i);
            if (c === CHAR_TAB) {
                i += 3;
                continue;
            }
            if (c !== CHAR_SPACE) {
                return false;
            }
        }
        return true;
    };
    Cursor.prototype.atWhitespace = function () {
        var c = this.currentCode();
        return c === CHAR_SPACE ||
            c === CHAR_TAB ||
            c === CHAR_LF ||
            c === CHAR_CR ||
            c === CHAR_FF;
    };
    Cursor.prototype.atTaint = function () {
        return this.currentCode() === 0 && this.pos < this.region.length;
    };
    Cursor.prototype.skipTainted = function () {
        while (this.atTaint()) {
            this.skip();
        }
        return this;
    };
    Cursor.prototype.skip = function (n) {
        if (n === void 0) { n = 1; }
        this._pos += n;
        return this;
    };
    Cursor.prototype.skipString = function (str) {
        if (this.at(str)) {
            this.skip(str.length);
        }
        return this;
    };
    Cursor.prototype.skipSpaces = function (count) {
        if (count === void 0) { count = -1; }
        var i = count;
        while (this.atSpace() || this.atTaint()) {
            if (i === 0) {
                break;
            }
            i -= 1;
            this.skip();
        }
        return this;
    };
    Cursor.prototype.skipNewLine = function () {
        var code = this.currentCode();
        if (code === CHAR_CR && this.peekCode() === CHAR_LF) {
            this.skip(2);
        }
        else if (code === CHAR_LF || code === CHAR_CR) {
            this.skip();
        }
        return this;
    };
    Cursor.prototype.skipNewLines = function () {
        var pos = -1;
        while (pos !== this.pos) {
            pos = this.pos;
            this.skipNewLine();
        }
        return this;
    };
    Cursor.prototype.skipWhitespaces = function () {
        var pos = -1;
        while (pos !== this.pos) {
            pos = this.pos;
            this.skipSpaces().skipNewLines();
        }
        return this;
    };
    Cursor.prototype.skipBlankLines = function () {
        while (this.hasCurrent()) {
            var pos = this.pos;
            this.skipSpaces();
            if (this.atNewLine() || !this.hasCurrent()) {
                this.skipNewLines();
            }
            else {
                this.set(pos);
                break;
            }
        }
        return this;
    };
    Cursor.prototype.skipToEol = function () {
        while (this.hasCurrent() && !this.atNewLine()) {
            this.skip();
        }
        return this;
    };
    Cursor.prototype.skipToEndOfBlock = function () {
        while (this.hasCurrent()) {
            if (this.atNewLine()) {
                var i = this.pos;
                if (this.skipNewLine().skipSpaces().atNewLine()) {
                    this.set(i);
                    break;
                }
            }
            this.skip();
        }
        return this;
    };
    Cursor.prototype.readUntil = function (newPos) {
        newPos = Math.max(this.pos, Math.min(newPos, this.region.length));
        var result = this.region.subRegion(this.pos, newPos);
        this.set(newPos);
        return result;
    };
    Cursor.prototype.readForward = function (relNewPos) {
        return this.readUntil(this.pos + relNewPos);
    };
    Cursor.prototype.readLine = function () {
        var start = this.pos;
        this.skipToEol().skipNewLine();
        return this.region.subRegion(start, this.pos);
    };
    Cursor.prototype.scan = function () {
        var seq = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            seq[_i] = arguments[_i];
        }
        var cur = this.clone();
        while (cur.hasCurrent()) {
            if (cur.atCode(CHAR_BACKSLASH)) {
                cur.skip(2);
                continue;
            }
            if (cur.atSeq.apply(cur, seq)) {
                return cur.pos;
            }
            cur.skip();
        }
        return null;
    };
    Cursor.prototype.scanLine = function () {
        var seq = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            seq[_i] = arguments[_i];
        }
        var cur = this.clone();
        while (cur.hasCurrent() && !cur.atNewLine()) {
            if (cur.atCode(CHAR_BACKSLASH)) {
                cur.skip(2);
                continue;
            }
            if (cur.atSeq.apply(cur, seq)) {
                return cur.pos;
            }
            cur.skip();
        }
        return null;
    };
    Cursor.prototype.debug = function () {
        var fmt = function (str) { return str.replace(/ /g, '·').replace(/\n/g, '↲\n'); };
        var prefix = this.region.substring(0, this.pos);
        var suffix = this.region.substring(this.pos);
        return '\u001b[33m' + fmt(prefix) + '\u001b[0m' +
            '\u001b[36m' + fmt(suffix) + '\u001b[0m';
    };
    return Cursor;
}());
exports.Cursor = Cursor;
//# sourceMappingURL=cursor.js.map