"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var cursor_1 = require("./cursor");
var node_1 = require("./node");
var exception_1 = require("./exception");
var region_1 = require("./region");
var Parser = (function () {
    function Parser(ctx, rules) {
        this.ctx = ctx;
        this.rules = rules;
    }
    Parser.prototype.parseString = function (str) {
        return this.parse(new region_1.Region(str));
    };
    Parser.prototype.parse = function (region) {
        var nodes = [];
        var cursor = new cursor_1.Cursor(region);
        while (cursor.hasCurrent()) {
            var node = this.parseSinglePass(cursor);
            if (node == null) {
                throw new exception_1.Exception({
                    code: 'InvalidParser',
                    message: 'Parser must emit a node on each pass',
                    details: {
                        parser: this,
                        cursor: cursor,
                    }
                });
            }
            nodes.push(node);
        }
        return new node_1.Node(region, nodes);
    };
    Parser.prototype.parseSinglePass = function (cursor) {
        for (var _i = 0, _a = this.rules; _i < _a.length; _i++) {
            var rule = _a[_i];
            var node = rule.parse(cursor);
            if (node) {
                return node;
            }
        }
        return null;
    };
    Parser.prototype.insertRuleBefore = function (predicate, newRule) {
        var i = this.rules.findIndex(function (r) { return predicate(r); });
        if (i > -1) {
            this.rules.splice(i, 0, newRule);
        }
        return this;
    };
    Parser.prototype.insertRuleAfter = function (predicate, newRule) {
        var i = this.rules.findIndex(function (r) { return predicate(r); });
        if (i > -1) {
            this.rules.splice(i + 1, 0, newRule);
        }
        return this;
    };
    Parser.prototype.replaceRule = function (predicate, newRule) {
        var i = this.rules.findIndex(function (r) { return predicate(r); });
        if (i > -1) {
            this.rules.splice(i, 1, newRule);
        }
        return this;
    };
    Parser.prototype.removeRule = function (predicate) {
        var i = this.rules.findIndex(function (r) { return predicate(r); });
        if (i > -1) {
            this.rules.splice(i, 0);
        }
        return this;
    };
    return Parser;
}());
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map