"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var exception_1 = require("./exception");
var context_1 = require("./context");
var Processor = (function () {
    function Processor(options) {
        if (options === void 0) { options = {}; }
        this.mainParserId = '';
        this.parserDefs = new Map();
        this.transforms = [];
        this.options = {};
        this.options = options;
    }
    Processor.prototype.toHtml = function (str) {
        var ctx = this.createContext();
        var ast = ctx.getMainParser().parseString(str);
        return ctx.render(ast);
    };
    Processor.prototype.createContext = function () {
        return new context_1.Context(this);
    };
    Processor.prototype.setMainParser = function (parserId) {
        this.mainParserId = parserId;
        return this;
    };
    Processor.prototype.defineParser = function (parserId, def) {
        this.parserDefs.set(parserId, def);
        return this;
    };
    Processor.prototype.getParserDef = function (parserId) {
        var def = this.parserDefs.get(parserId);
        if (!def) {
            throw new exception_1.Exception({
                code: 'ParserNotFound',
                message: "Parser \"" + parserId + "\" not found, please update Processor accordingly"
            });
        }
        return def;
    };
    Processor.prototype.getMainParserDef = function () {
        if (!this.mainParserId) {
            throw new exception_1.Exception({
                code: 'ProcessorNotConfigured',
                message: "Main parser not specified, please update Processor accordingly",
            });
        }
        return this.getParserDef(this.mainParserId);
    };
    Processor.prototype.transform = function (fn) {
        this.transforms.push(fn);
        return this;
    };
    return Processor;
}());
exports.Processor = Processor;
//# sourceMappingURL=processor.js.map