"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Region = (function () {
    function Region(str, start, end) {
        if (start === void 0) { start = 0; }
        if (end === void 0) { end = str.length; }
        this.str = str;
        this.start = start;
        this.end = end;
        this.length = this.end - this.start;
    }
    Region.prototype.charAt = function (i) {
        if (i < 0 || i >= this.length) {
            return '';
        }
        return this.str.charAt(this.start + i);
    };
    Region.prototype.charCodeAt = function (i) {
        return this.str.charCodeAt(this.start + i);
    };
    Region.prototype.substring = function (start, end) {
        if (end === void 0) { end = this.length; }
        start = Math.max(0, start);
        end = Math.min(this.length, end);
        return this.str.substring(this.start + start, this.start + end);
    };
    Region.prototype.subRegion = function (start, end) {
        if (end === void 0) { end = this.length; }
        start = Math.max(0, start);
        end = Math.min(this.length, end);
        return new Region(this.str, this.start + start, this.start + end);
    };
    Region.prototype.toString = function () {
        return this.substring(0);
    };
    Region.prototype.taint = function (from, to) {
        var taint = [from, to];
        if (!isTaintInRegion(this.start, this.end, taint)) {
            return this;
        }
        return new TaintedRegion(this.str, this.start, this.end, [taint]);
    };
    Region.prototype.taintRelative = function (from, to) {
        return this.taint(this.start + from, this.start + to);
    };
    Region.prototype.taintRegion = function (subRegion) {
        return this.taint(subRegion.start, subRegion.end);
    };
    Region.prototype.untaint = function () {
        return [this];
    };
    return Region;
}());
exports.Region = Region;
var TaintedRegion = (function (_super) {
    __extends(TaintedRegion, _super);
    function TaintedRegion(str, start, end, taints) {
        if (start === void 0) { start = 0; }
        if (end === void 0) { end = str.length; }
        if (taints === void 0) { taints = []; }
        var _this = _super.call(this, str, start, end) || this;
        _this.str = str;
        _this.start = start;
        _this.end = end;
        _this.taints = taints;
        return _this;
    }
    TaintedRegion.prototype.charAt = function (i) {
        if (i < 0 || i >= this.length) {
            return '';
        }
        var index = this.start + i;
        for (var _i = 0, _a = this.taints; _i < _a.length; _i++) {
            var taint = _a[_i];
            if (index >= taint[0] && index < taint[1]) {
                return '';
            }
        }
        return this.str.charAt(this.start + i);
    };
    TaintedRegion.prototype.charCodeAt = function (i) {
        var index = this.start + i;
        for (var _i = 0, _a = this.taints; _i < _a.length; _i++) {
            var taint = _a[_i];
            if (index >= taint[0] && index < taint[1]) {
                return 0;
            }
        }
        return this.str.charCodeAt(this.start + i);
    };
    TaintedRegion.prototype.substring = function (start, end) {
        if (end === void 0) { end = this.length; }
        start = Math.max(0, start);
        end = Math.min(this.length, end);
        var result = '';
        var indexes = this.untaintedIndexes(start, end);
        for (var _i = 0, indexes_1 = indexes; _i < indexes_1.length; _i++) {
            var _a = indexes_1[_i], s = _a[0], e = _a[1];
            result += this.str.substring(s, e);
        }
        return result;
    };
    TaintedRegion.prototype.subRegion = function (start, end) {
        if (end === void 0) { end = this.length; }
        start = this.start + Math.max(0, start);
        end = this.start + Math.min(this.length, end);
        var taints = this.taints.filter(function (t) { return isTaintInRegion(start, end, t); });
        if (taints.length > 0) {
            return new TaintedRegion(this.str, start, end, taints);
        }
        return new Region(this.str, start, end);
    };
    TaintedRegion.prototype.taint = function (from, to) {
        var taint = [from, to];
        if (!isTaintInRegion(this.start, this.end, taint)) {
            return this;
        }
        var taints = this.taints.concat([taint]).sort(function (a, b) { return a[0] > b[0] ? 1 : -1; });
        return new TaintedRegion(this.str, this.start, this.end, taints);
    };
    TaintedRegion.prototype.taintRelative = function (from, to) {
        return this.taint(this.start + from, this.start + to);
    };
    TaintedRegion.prototype.taintRegion = function (subRegion) {
        return this.taint(subRegion.start, subRegion.end);
    };
    TaintedRegion.prototype.untaint = function () {
        var result = [];
        var indexes = this.untaintedIndexes(0, this.length);
        for (var _i = 0, indexes_2 = indexes; _i < indexes_2.length; _i++) {
            var _a = indexes_2[_i], s = _a[0], e = _a[1];
            var region = new Region(this.str, s, e);
            result.push(region);
        }
        return result;
    };
    TaintedRegion.prototype.untaintedIndexes = function (start, end) {
        var result = [];
        start = this.start + Math.max(0, start);
        end = this.start + Math.min(this.length, end);
        var s = start;
        for (var _i = 0, _a = this.taints; _i < _a.length; _i++) {
            var taint = _a[_i];
            if (taint[1] < s) {
                continue;
            }
            if (taint[0] <= s) {
                s = taint[1];
                continue;
            }
            var e = Math.min(end, taint[0]);
            if (s >= e) {
                break;
            }
            result.push([s, e]);
            s = taint[1];
        }
        if (s < end) {
            result.push([s, end]);
        }
        return result;
    };
    return TaintedRegion;
}(Region));
exports.TaintedRegion = TaintedRegion;
function isTaintInRegion(regionStart, regionEnd, taint) {
    return taint[0] < regionEnd && taint[1] > regionStart;
}
//# sourceMappingURL=region.js.map