"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("../../core");
var selector_1 = require("../../util/selector");
var SELECTOR_LOOKUP_LIMIT = 120;
var CHAR_SPACE = core_1.constants.CHAR_SPACE, CHAR_TAB = core_1.constants.CHAR_TAB, CHAR_BACKSLASH = core_1.constants.CHAR_BACKSLASH, CHAR_CURLY_LEFT = core_1.constants.CHAR_CURLY_LEFT, CHAR_CURLY_RIGHT = core_1.constants.CHAR_CURLY_RIGHT, CHAR_HASH = core_1.constants.CHAR_HASH, CHAR_PERIOD = core_1.constants.CHAR_PERIOD;
var BlockRule = (function (_super) {
    __extends(BlockRule, _super);
    function BlockRule() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.lineStartPos = 0;
        _this.indent = 0;
        _this.selector = null;
        return _this;
    }
    BlockRule.prototype.allowsMultipleSelectors = function () {
        return false;
    };
    BlockRule.prototype.parseAt = function (cursor) {
        cursor.skipBlankLines();
        this.countBlockIndent(cursor);
        var region = this.scanBlock(cursor.clone());
        if (region == null) {
            return null;
        }
        cursor.set(region.end - cursor.region.start);
        this.selector = this.captureSelector(region, this.allowsMultipleSelectors());
        if (this.selector) {
            region = region.taintRegion(this.selector.region);
        }
        return this.parseSubRegion(region);
    };
    BlockRule.prototype.countBlockIndent = function (cursor) {
        this.indent = 0;
        while (cursor.hasCurrent()) {
            var c = cursor.currentCode();
            if (c === CHAR_SPACE) {
                this.indent += 1;
                cursor.skip();
            }
            else if (c === CHAR_TAB) {
                this.indent += 4;
                cursor.skip();
            }
            else {
                break;
            }
        }
    };
    BlockRule.prototype.parseInlineContent = function (region) {
        var parser = this.ctx.getParser('inline');
        var result = [];
        var regions = region.untaint();
        for (var _i = 0, regions_1 = regions; _i < regions_1.length; _i++) {
            var region_1 = regions_1[_i];
            var ast = parser.parse(region_1);
            result.push.apply(result, ast.children);
        }
        return result;
    };
    BlockRule.prototype.parseBlockContent = function (region) {
        var parser = this.ctx.getParser('block');
        var ast = parser.parse(region);
        return ast.children;
    };
    BlockRule.prototype.captureSelector = function (region, allowMultiple) {
        if (allowMultiple === void 0) { allowMultiple = false; }
        return this.parseSelectorAt(new core_1.Cursor(region), allowMultiple);
    };
    BlockRule.prototype.parseSelectorAt = function (cursor, allowMultiple) {
        if (allowMultiple === void 0) { allowMultiple = false; }
        var i = 0;
        while (cursor.hasCurrent() && !cursor.atNewLine() && i < SELECTOR_LOOKUP_LIMIT) {
            i++;
            if (cursor.atCode(CHAR_BACKSLASH)) {
                cursor.skip(2);
                continue;
            }
            if (!cursor.atCode(CHAR_CURLY_LEFT)) {
                cursor.skip();
                continue;
            }
            var start = cursor.pos;
            cursor.skip();
            var id = this.parseSelectorComponent(CHAR_HASH, cursor) || '';
            var classList = [];
            while (true) {
                var cl = this.parseSelectorComponent(CHAR_PERIOD, cursor);
                if (cl) {
                    classList.push(cl);
                    continue;
                }
                break;
            }
            if (!cursor.atCode(CHAR_CURLY_RIGHT)) {
                continue;
            }
            cursor.skip().skipSpaces();
            var found = cursor.atNewLine() || !cursor.hasCurrent();
            if (!found && allowMultiple) {
                var nextSelector = this.parseSelectorAt(cursor.clone(), allowMultiple);
                found = nextSelector != null;
            }
            if (found) {
                var region = cursor.subRegion(start, cursor.pos);
                var node = new selector_1.Selector(region, id, classList);
                return node;
            }
        }
        return null;
    };
    BlockRule.prototype.parseSelectorComponent = function (marker, cursor) {
        if (!cursor.atCode(marker)) {
            return null;
        }
        cursor.skip();
        var start = cursor.pos;
        while (cursor.hasCurrent() && cursor.atIdentifier()) {
            cursor.skip();
        }
        var region = cursor.subRegion(start, cursor.pos);
        return region.length ? region.toString() : null;
    };
    return BlockRule;
}(core_1.Rule));
exports.BlockRule = BlockRule;
//# sourceMappingURL=block.js.map