"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("../../core");
var nodes_1 = require("../../nodes");
var block_1 = require("./block");
var ListRule = (function (_super) {
    __extends(ListRule, _super);
    function ListRule(ctx, options) {
        var _this = _super.call(this, ctx) || this;
        _this.tagName = options.tagName;
        _this.marker = options.marker;
        return _this;
    }
    ListRule.prototype.allowsMultipleSelectors = function () {
        return true;
    };
    ListRule.prototype.skipMarker = function (cursor) {
        cursor.skipString(this.marker);
    };
    ListRule.prototype.isAtMarker = function (cursor) {
        var start = cursor.pos;
        this.skipMarker(cursor);
        var result = cursor.pos > start;
        cursor.set(start);
        return result;
    };
    ListRule.prototype.scanBlock = function (cursor) {
        if (!cursor.at(this.marker)) {
            return null;
        }
        var start = cursor.pos;
        while (cursor.hasCurrent()) {
            cursor.skipToEndOfBlock();
            var endOfBlock = cursor.pos;
            cursor.skipBlankLines();
            var startOfLine = cursor.pos;
            if (!cursor.atSpaces(this.indent)) {
                cursor.set(startOfLine);
                break;
            }
            cursor.skip(this.indent);
            var isSibling = this.isAtMarker(cursor);
            var isSubBlock = cursor.atSpace();
            if (!isSibling && !isSubBlock) {
                cursor.set(endOfBlock);
                break;
            }
        }
        return cursor.subRegion(start, cursor.pos);
    };
    ListRule.prototype.parseSubRegion = function (region) {
        var regions = [];
        var cursor = new core_1.Cursor(region);
        this.skipMarker(cursor);
        var hasBlocks = false;
        var start = cursor.pos;
        while (cursor.hasCurrent()) {
            cursor.skipToEol().skipNewLine();
            if (cursor.atBlankLine()) {
                cursor.skipBlankLines();
                if (cursor.hasCurrent()) {
                    hasBlocks = true;
                }
            }
            if (cursor.atSpaces(this.indent)) {
                cursor.skip(this.indent);
            }
            if (this.isAtMarker(cursor)) {
                var region_1 = cursor.subRegion(start, cursor.pos);
                regions.push(region_1);
                this.skipMarker(cursor);
                start = cursor.pos;
            }
        }
        var lastRegion = cursor.subRegion(start, cursor.pos);
        regions.push(lastRegion);
        var children = this.parseLiRegions(regions, hasBlocks);
        return new nodes_1.HtmlElementNode(lastRegion, children, this.tagName, this.selector);
    };
    ListRule.prototype.parseLiRegions = function (regions, hasBlocks) {
        var nodes = [];
        for (var _i = 0, regions_1 = regions; _i < regions_1.length; _i++) {
            var region = regions_1[_i];
            var cursor = new core_1.Cursor(region);
            var selector = this.parseSelectorAt(cursor.clone(), true);
            if (selector) {
                cursor = new core_1.Cursor(region.taintRegion(selector.region));
            }
            var li = hasBlocks ?
                this.parseBlockLi(cursor, selector) :
                this.parseTerseLi(cursor, selector);
            nodes.push(li);
        }
        return nodes;
    };
    ListRule.prototype.parseTerseLi = function (cursor, selector) {
        var children = [];
        var listParser = this.ctx.getParser('list');
        var inlineStart = cursor.pos;
        cursor.skipToEol().skipNewLine();
        while (cursor.hasCurrent()) {
            var pos = cursor.pos;
            var list = listParser.parseSinglePass(cursor);
            if (list) {
                if (inlineStart !== pos) {
                    var region = cursor.subRegion(inlineStart, pos);
                    children.push.apply(children, this.parseInlineContent(region));
                }
                children.push(list);
                inlineStart = cursor.pos;
            }
            else {
                cursor.skipToEol().skipBlankLines();
            }
        }
        if (inlineStart !== cursor.pos) {
            var region = cursor.subRegion(inlineStart, cursor.pos);
            children.push.apply(children, this.parseInlineContent(region));
        }
        return new nodes_1.HtmlElementNode(cursor.region, children, 'li', selector);
    };
    ListRule.prototype.parseBlockLi = function (cursor, selector) {
        var children = this.parseBlockContent(cursor.region);
        return new nodes_1.HtmlElementNode(cursor.region, children, 'li', selector);
    };
    return ListRule;
}(block_1.BlockRule));
exports.ListRule = ListRule;
var NumberedListRule = (function (_super) {
    __extends(NumberedListRule, _super);
    function NumberedListRule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NumberedListRule.prototype.skipMarker = function (cursor) {
        var startsWithDigit = false;
        while (cursor.atDigit()) {
            startsWithDigit = true;
            cursor.skip();
        }
        if (startsWithDigit) {
            cursor.skipString('. ');
        }
    };
    return NumberedListRule;
}(ListRule));
exports.NumberedListRule = NumberedListRule;
//# sourceMappingURL=list.js.map