"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("../../core");
var escape_1 = require("../../util/escape");
var media_1 = require("../../util/media");
var CHAR_SQUARE_LEFT = core_1.constants.CHAR_SQUARE_LEFT, CHAR_SQUARE_RIGHT = core_1.constants.CHAR_SQUARE_RIGHT, CHAR_BACKSLASH = core_1.constants.CHAR_BACKSLASH, CHAR_EXCLAMATION = core_1.constants.CHAR_EXCLAMATION, CHAR_PAREN_LEFT = core_1.constants.CHAR_PAREN_LEFT, CHAR_PAREN_RIGHT = core_1.constants.CHAR_PAREN_RIGHT;
var LinkRule = (function (_super) {
    __extends(LinkRule, _super);
    function LinkRule(ctx) {
        var _this = _super.call(this, ctx) || this;
        _this.ctx = ctx;
        return _this;
    }
    LinkRule.prototype.parseAt = function (cursor) {
        if (!cursor.atCode(CHAR_SQUARE_LEFT)) {
            return null;
        }
        var regionStart = cursor.pos;
        cursor.skip();
        var inlineParser = this.ctx.getParser('inline');
        var textStart = cursor.pos;
        var textEnd = this.findClosingMarker(cursor.clone());
        if (textEnd == null) {
            return null;
        }
        cursor.set(textEnd + 1);
        var textRegion = cursor.subRegion(textStart, textEnd);
        var children = inlineParser.parse(textRegion).children;
        return this.tryInlineLink(children, cursor, regionStart)
            || this.tryRefLink(children, cursor, regionStart);
    };
    LinkRule.prototype.findClosingMarker = function (cursor) {
        var nesting = 0;
        while (cursor.hasCurrent()) {
            if (cursor.atCode(CHAR_SQUARE_RIGHT) && !nesting) {
                return cursor.pos;
            }
            if (cursor.atCode(CHAR_BACKSLASH)) {
                cursor.skip(2);
                continue;
            }
            if (cursor.atSeq(CHAR_EXCLAMATION, CHAR_SQUARE_LEFT)) {
                cursor.skip(2);
                nesting += 1;
                continue;
            }
            if (cursor.atSeq(CHAR_SQUARE_RIGHT, CHAR_SQUARE_LEFT) && nesting > 0) {
                cursor.skip(2);
                continue;
            }
            if (cursor.atCode(CHAR_SQUARE_RIGHT)) {
                cursor.skip();
                nesting -= 1;
                continue;
            }
            cursor.skip();
        }
        return null;
    };
    LinkRule.prototype.tryInlineLink = function (children, cursor, regionStart) {
        if (!cursor.atCode(CHAR_PAREN_LEFT)) {
            return null;
        }
        cursor.skip();
        var hrefStart = cursor.pos;
        var hrefEnd = cursor.scan(CHAR_PAREN_RIGHT);
        if (hrefEnd == null) {
            return null;
        }
        cursor.set(hrefEnd + 1);
        var href = cursor.subRegion(hrefStart, hrefEnd).toString();
        var region = cursor.subRegion(regionStart, cursor.pos);
        var id = this.ctx.getNextInlineId();
        this.ctx.mediaIds.add(id);
        this.ctx.resolvedMedia.set(id, { href: href });
        return new LinkNode(region, children, id);
    };
    LinkRule.prototype.tryRefLink = function (children, cursor, regionStart) {
        if (!cursor.atCode(CHAR_SQUARE_LEFT)) {
            return null;
        }
        cursor.skip();
        var idStart = cursor.pos;
        var idEnd = cursor.scan(CHAR_SQUARE_RIGHT);
        if (idEnd == null) {
            return null;
        }
        cursor.set(idEnd + 1);
        var id = cursor.subRegion(idStart, idEnd).toString();
        var region = cursor.subRegion(regionStart, cursor.pos);
        this.ctx.mediaIds.add(id);
        return new LinkNode(region, children, id);
    };
    return LinkRule;
}(core_1.Rule));
exports.LinkRule = LinkRule;
var HeadlessLinkRule = (function (_super) {
    __extends(HeadlessLinkRule, _super);
    function HeadlessLinkRule(ctx) {
        var _this = _super.call(this, ctx) || this;
        _this.ctx = ctx;
        return _this;
    }
    HeadlessLinkRule.prototype.parseAt = function (cursor) {
        if (!cursor.atSeq(CHAR_SQUARE_LEFT, CHAR_SQUARE_LEFT)) {
            return null;
        }
        var regionStart = cursor.pos;
        cursor.skip(2);
        var idStart = cursor.pos;
        var idEnd = cursor.scan(CHAR_SQUARE_RIGHT, CHAR_SQUARE_RIGHT);
        if (idEnd == null) {
            return null;
        }
        cursor.set(idEnd + 2);
        var id = cursor.subRegion(idStart, idEnd).toString();
        this.ctx.mediaIds.add(id);
        var region = cursor.subRegion(regionStart, cursor.pos);
        return new LinkNode(region, [], id, true);
    };
    return HeadlessLinkRule;
}(core_1.Rule));
exports.HeadlessLinkRule = HeadlessLinkRule;
var LinkNode = (function (_super) {
    __extends(LinkNode, _super);
    function LinkNode(region, children, id, isHeadless) {
        if (isHeadless === void 0) { isHeadless = false; }
        var _this = _super.call(this, region, children) || this;
        _this.id = id;
        _this.isHeadless = isHeadless;
        return _this;
    }
    LinkNode.prototype.resolveMedia = function (ctx) {
        return ctx.resolvedMedia.get(this.id) || null;
    };
    LinkNode.prototype.renderContent = function (ctx) {
        var _a;
        if (this.isHeadless) {
            var media = this.resolveMedia(ctx);
            return ((_a = media) === null || _a === void 0 ? void 0 : _a.title) || '';
        }
        return ctx.renderChildren(this);
    };
    LinkNode.prototype.render = function (ctx) {
        var media = this.resolveMedia(ctx);
        if (media == null) {
            return this.renderContent(ctx);
        }
        var src = media_1.parseHref(media.href).src;
        var buffer = '<a';
        buffer += " href=\"" + escape_1.escapeHtml(src) + "\"";
        if (media.title) {
            buffer += " title=\"" + escape_1.escapeHtml(media.title) + "\"";
        }
        if (ctx.isExternalLink(media)) {
            buffer += ' target="_blank"';
        }
        buffer += '>';
        buffer += this.renderContent(ctx);
        buffer += '</a>';
        return buffer;
    };
    return LinkNode;
}(core_1.Node));
exports.LinkNode = LinkNode;
//# sourceMappingURL=link.js.map