"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("../../core");
var escape_1 = require("../../util/escape");
var media_1 = require("../../util/media");
var CHAR_SQUARE_LEFT = core_1.constants.CHAR_SQUARE_LEFT, CHAR_SQUARE_RIGHT = core_1.constants.CHAR_SQUARE_RIGHT, CHAR_EXCLAMATION = core_1.constants.CHAR_EXCLAMATION, CHAR_PAREN_LEFT = core_1.constants.CHAR_PAREN_LEFT, CHAR_PAREN_RIGHT = core_1.constants.CHAR_PAREN_RIGHT;
var MediaRule = (function (_super) {
    __extends(MediaRule, _super);
    function MediaRule(ctx) {
        var _this = _super.call(this, ctx) || this;
        _this.ctx = ctx;
        return _this;
    }
    MediaRule.prototype.parseAt = function (cursor) {
        if (!(cursor.atCode(CHAR_EXCLAMATION) && cursor.atCode(CHAR_SQUARE_LEFT, 1))) {
            return null;
        }
        var regionStart = cursor.pos;
        cursor.skip(2);
        var textStart = cursor.pos;
        var textEnd = cursor.scan(CHAR_SQUARE_RIGHT);
        if (textEnd == null) {
            return null;
        }
        cursor.set(textEnd + 1);
        var text = cursor.subRegion(textStart, textEnd).toString();
        return this.tryInlineImg(cursor, text, regionStart)
            || this.tryRefImg(cursor, text, regionStart);
    };
    MediaRule.prototype.tryInlineImg = function (cursor, text, regionStart) {
        if (!cursor.atCode(CHAR_PAREN_LEFT)) {
            return null;
        }
        cursor.skip();
        var hrefStart = cursor.pos;
        var hrefEnd = cursor.scan(CHAR_PAREN_RIGHT);
        if (hrefEnd == null) {
            return null;
        }
        cursor.set(hrefEnd + 1);
        var id = this.ctx.getNextInlineId();
        var href = cursor.subRegion(hrefStart, hrefEnd).toString();
        var region = cursor.subRegion(regionStart, cursor.pos);
        this.ctx.mediaIds.add(id);
        this.ctx.resolvedMedia.set(id, { href: href });
        return new MediaNode(region, id, text);
    };
    MediaRule.prototype.tryRefImg = function (cursor, text, regionStart) {
        if (!cursor.atCode(CHAR_SQUARE_LEFT)) {
            return null;
        }
        cursor.skip();
        var idStart = cursor.pos;
        var idEnd = cursor.scan(CHAR_SQUARE_RIGHT);
        if (idEnd == null) {
            return null;
        }
        var id = cursor.subRegion(idStart, idEnd).toString();
        cursor.set(idEnd + 1);
        var region = cursor.subRegion(regionStart, cursor.pos);
        this.ctx.mediaIds.add(id);
        return new MediaNode(region, id, text);
    };
    return MediaRule;
}(core_1.Rule));
exports.MediaRule = MediaRule;
var HeadlessMediaRule = (function (_super) {
    __extends(HeadlessMediaRule, _super);
    function HeadlessMediaRule(ctx) {
        var _this = _super.call(this, ctx) || this;
        _this.ctx = ctx;
        return _this;
    }
    HeadlessMediaRule.prototype.parseAt = function (cursor) {
        if (!cursor.atSeq(CHAR_EXCLAMATION, CHAR_SQUARE_LEFT, CHAR_SQUARE_LEFT)) {
            return null;
        }
        var regionStart = cursor.pos;
        cursor.skip(3);
        var idStart = cursor.pos;
        var idEnd = cursor.scan(CHAR_SQUARE_RIGHT, CHAR_SQUARE_RIGHT);
        if (idEnd == null) {
            return null;
        }
        cursor.set(idEnd + 2);
        var id = cursor.subRegion(idStart, idEnd).toString();
        this.ctx.mediaIds.add(id);
        var region = cursor.subRegion(regionStart, cursor.pos);
        return new MediaNode(region, id);
    };
    return HeadlessMediaRule;
}(core_1.Rule));
exports.HeadlessMediaRule = HeadlessMediaRule;
var MediaNode = (function (_super) {
    __extends(MediaNode, _super);
    function MediaNode(region, id, text) {
        if (text === void 0) { text = ''; }
        var _this = _super.call(this, region, []) || this;
        _this.id = id;
        _this.text = text;
        return _this;
    }
    MediaNode.prototype.resolveMedia = function (ctx) {
        return ctx.resolvedMedia.get(this.id) || null;
    };
    MediaNode.prototype.render = function (ctx) {
        var media = this.resolveMedia(ctx);
        if (media == null) {
            return '';
        }
        if (media.customRender) {
            return media.customRender(this, ctx);
        }
        var _a = media_1.parseHref(media.href), src = _a.src, width = _a.width, height = _a.height;
        var alt = this.text || media.title;
        var title = media.title || this.text;
        var buffer = '<img';
        buffer += " src=\"" + escape_1.escapeHtml(src) + "\"";
        if (alt) {
            buffer += " alt=\"" + escape_1.escapeHtml(alt) + "\"";
        }
        if (title) {
            buffer += " title=\"" + escape_1.escapeHtml(title) + "\"";
        }
        if (width != null) {
            buffer += " width=\"" + width + "\"";
        }
        if (height != null) {
            buffer += " height=\"" + height + "\"";
        }
        if (ctx.isLazy(media)) {
            buffer += ' loading="lazy"';
        }
        buffer += '/>';
        return buffer;
    };
    return MediaNode;
}(core_1.Node));
exports.MediaNode = MediaNode;
//# sourceMappingURL=media.js.map