"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("../core");
var CHAR_LT = core_1.constants.CHAR_LT, CHAR_GT = core_1.constants.CHAR_GT, CHAR_SLASH = core_1.constants.CHAR_SLASH, CHAR_COLON = core_1.constants.CHAR_COLON;
function matchHtmlTag(cursor) {
    if (!cursor.atCode(CHAR_LT)) {
        return null;
    }
    var start = cursor.pos;
    cursor.skip();
    var closingTag = false;
    if (cursor.atCode(CHAR_SLASH)) {
        cursor.skip();
        closingTag = true;
    }
    var tagNameStart = cursor.pos;
    if (!cursor.atLatin()) {
        return null;
    }
    while (cursor.atIdentifier() || cursor.atCode(CHAR_COLON)) {
        cursor.skip();
    }
    var tagName = cursor.subRegion(tagNameStart, cursor.pos)
        .toString()
        .toLowerCase();
    if (closingTag) {
        cursor.skipWhitespaces();
        if (!cursor.atCode(CHAR_GT)) {
            return null;
        }
        cursor.skip();
        var region = cursor.subRegion(start, cursor.pos);
        return { type: HtmlTagType.CLOSING, tagName: tagName, region: region };
    }
    if (!cursor.atWhitespace()) {
        var type = matchTagEnd(cursor);
        if (!type) {
            return null;
        }
        var region = cursor.subRegion(start, cursor.pos);
        return { type: type, tagName: tagName, region: region };
    }
    cursor.skipWhitespaces();
    while (cursor.hasCurrent()) {
        var type = matchTagEnd(cursor);
        if (type) {
            var region = cursor.subRegion(start, cursor.pos);
            return { type: type, tagName: tagName, region: region };
        }
        cursor.skip();
    }
    return null;
}
exports.matchHtmlTag = matchHtmlTag;
function matchTagEnd(cursor) {
    if (cursor.atCode(CHAR_GT)) {
        cursor.skip();
        return HtmlTagType.OPENING;
    }
    if (cursor.atCode(CHAR_SLASH) && cursor.atCode(CHAR_GT, 1)) {
        cursor.skip(2);
        return HtmlTagType.SELF_CLOSING;
    }
    return null;
}
var HtmlTagType;
(function (HtmlTagType) {
    HtmlTagType[HtmlTagType["OPENING"] = 1] = "OPENING";
    HtmlTagType[HtmlTagType["CLOSING"] = 2] = "CLOSING";
    HtmlTagType[HtmlTagType["SELF_CLOSING"] = 3] = "SELF_CLOSING";
})(HtmlTagType = exports.HtmlTagType || (exports.HtmlTagType = {}));
//# sourceMappingURL=html.js.map